/*
 * This file is part of the Owl Positioning System (OwlPS) project.
 * It is subject to the copyright notice and license terms in the
 * COPYRIGHT.t2t file found in the top-level directory of this
 * distribution and at
 *   http://code.lm7.fr/p/owlps/source/tree/master/COPYRIGHT.t2t
 * No part of the OwlPS Project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms
 * contained in the COPYRIGHT.t2t file; the COPYRIGHT.t2t file must be
 * distributed along with this file, either separately or by replacing
 * this notice by the COPYRIGHT.t2t file's contents.
 */


#include "resultlist.hh"
#include "request.hh"
#include "mobile.hh"
#include "posexcept.hh"

#include <sstream>

using namespace std ;



/* *** Constructors *** */


ResultList::~ResultList()
{
  results.clear() ;
}



/* *** Read accessors *** */


const Result& ResultList::get_result_for_algo(const string &algo) const
{
  for (auto i = results.begin() ; i != results.end() ; ++i)
    if (i->get_algorithm() == algo)
      return *i ;

  throw element_not_found("No results for algorithm \""+ algo +"\"") ;
}



/* *** Operators *** */


ResultList& ResultList::operator=(const ResultList &source)
{
  if (this == &source)
    return *this ;

  request = source.request ;
  results = source.results ;

  return *this ;
}


bool ResultList::operator==(const ResultList &source) const
{
  return
    request == source.request &&
    results == source.results ;
}



/* *** Conversion accessors *** */


/**
 * This function creates a CSV string from the Result's data.
 *
 * The format used is the following:
 *   Mobile_MAC;Request_type;Timestamp;[Algorithm_1];…;[Algorithm_n]
 *
 * The CSV format for the [Algorithm_i] strings is documented in
 * Result::to_csv().
 *
 * @returns The results as a CSV string, *without* a trailing '\\n'.
 */
const string ResultList::to_csv() const
{
  ostringstream csv_line ;

  if (request)
    {
      if (request->get_mobile())
        csv_line << request->get_mobile()->get_mac_addr() ;
      csv_line
          << ';' << static_cast<uint_fast16_t>(request->get_type())
          << ';' << request->get_time_sent() ;
    }
  else
    csv_line << ";;;" ;

  for (auto r = results.begin() ; r != results.end() ; ++r)
    csv_line << ';' << r->to_csv() ;

  return csv_line.str() ;
}



ostream& operator<<(ostream &os, const ResultList &r)
{
  if (!r.request)
    os << "For an unknown request:\n" ;
  else
    os << "For the following request: " << *(r.request) << '\n' ;

  for (auto res = r.results.begin() ; res != r.results.end() ; ++res)
    os << *res << '\n' ;

  return os ;
}
